clear; close all; clc;

% This program performs Monte-carlo simulation on the SER
% performance of LDR detectors in MPSK constellation.
% 
%   written by Liu Huikang.
%   last updated on Nov. 25, 2013

%%
N = 8; M =8; % N - number transmit symbols, M - receive dimension.
MPSK_size=2; u=MPSK_size; %MPSK size
SNR = 6:2:10;   %SNR in dB
max_no_simulations =1; % No. of Monte-carlo simulations used


%-------------------------
sigma_snr = sqrt( N*10 .^ ( - (SNR) / 10 ) ); % MMSE coefficient.

Es=2/3*(MPSK_size-1); 
Es=1;
%detected symbols
sc_nld = zeros(N,1); 
sc_mmse_ld = zeros(N,1); 
sc_ldr_ld = zeros(N,1); 
sc_ml = zeros(N,1); 
sc_lra_zf_df = zeros(N,1); 
sc_lra_mmse_df = zeros(N,1); 
sc_ldr_lra_df = zeros(N,1); 
sc_zf_df = zeros(N,1); 

s_nld = zeros(2*N,1); 
s_mmse_ld = zeros(2*N,1); 
s_ldr_ld = zeros(2*N,1); 
s_ml = zeros(2*N,1); 
s_lra_zf_df = zeros(2*N,1); 
s_lra_mmse_df = zeros(2*N,1); 
s_ldr_lra_df = zeros(2*N,1); 
s_zf_df = zeros(2*N,1); 

s_gpm = zeros(2*N,1); 


%variables for storing the SERs
ser_nld = zeros(length(SNR),1); 
ser_mmse_ld = zeros(length(SNR),1); 
ser_ldr_ld = zeros(length(SNR),1); 
ser_ml = zeros(length(SNR),1); 
ser_lra_zf_df = zeros(length(SNR),1); 
ser_lra_mmse_df = zeros(length(SNR),1); 
ser_ldr_lra_df = zeros(length(SNR),1); 
ser_zf_df = zeros(length(SNR),1); 

ser_gpm = zeros(length(SNR),1);

for snr_pt = 1 : length(SNR)
    wb= waitbar(0,[int2str(snr_pt) '/' int2str(length(SNR))]);
    max_no_simulations =4*SNR(snr_pt)*1e4;
    for simulation_no = 1 : max_no_simulations
        waitbar(simulation_no/max_no_simulations,wb);
        
        %-----------set up channels, tx & rx signals , etc-------------
        % Generate QAM symbols
        s=randi(u,N,1); s_c = exp((1i*2*pi.*s)./u);
        
        % generate the random channel
        H_c = (1/sqrt(2))* (randn(M,N) + 1i*randn(M,N));
        % generate the output vector
        y_c = H_c*s_c + (sigma_snr(snr_pt)*sqrt(Es)/sqrt(2))*(randn(M,1)+1i*randn(M,1));
        
        rho = sigma_snr(snr_pt)*sqrt(Es);
        %--------detectors------------------
        
        %ZF DF
        [sc_zf_df] = zf_df(y_c,H_c,u);
        
%         %LRA-ZF-DF
%         s_lra_zf_df=lra_df_regu(y,H,zeros(2*N),u);
%         
%         %LRA-MMSE-DF
%         s_lra_mmse_df=lra_df_regu(y,H,diag(sigma_snr(snr_pt)^2*ones(2*N,1)),u);
%         
%         %LDR-LRA-DF
%         s_ldr_lra_df=ldr_lra_df(y,H,u,LDR_iter);
        


%         %naive lattice decoding
%         s_nld=ld_regu(y,H,zeros(2*N),u);
%          
%         %MMSE lattice decoding
       sc_mmse_ld=ld_regu(y_c,H_c,diag(sigma_snr(snr_pt)^2*ones(N,1)),u);
          %ML
       s_ml=ml(y_c,H_c,u,rho);
          %GPM
%           stepsize=0.75+SNR(snr_pt)^2/6000;
%           if stepsize>1
%               stepsize=1;
%           end
stepsize=1;
       [s_gpm,iternum]=GPM(y_c,H_c,u,rho,s_ml,stepsize);

%           
%         %LDR LD
%         s_ldr_ld=ldr_ld(y,H,u,LDR_iter);
%          

        
        
        
        %-------count the error-------------
        
        
        
        %count errors
                sumd=0;
        for iterea=1:N
            if norm(sc_zf_df(iterea)-s_c(iterea),1)>1e-2
                sumd=sumd+1;
            end
        end
            
        ser_zf_df(snr_pt)=ser_zf_df(snr_pt)+sumd;
        if simulation_no == 1
            ser_lra_zf_df(snr_pt)=iternum;
        elseif ser_lra_zf_df(snr_pt)<iternum;
            ser_lra_zf_df(snr_pt)=iternum;
        end
        ser_lra_mmse_df(snr_pt)=ser_lra_mmse_df(snr_pt)+sum(sc_lra_mmse_df~=s_c);
        ser_nld(snr_pt)=ser_nld(snr_pt)+iternum;
        
                       sumd=0;
        for iterea=1:N
            if norm(sc_mmse_ld(iterea)-s_c(iterea),1)>1e-2
                sumd=sumd+1;
            end
        end
        ser_mmse_ld(snr_pt)=ser_mmse_ld(snr_pt)+sumd;
        
            sumk=0;
        for iterea=1:N
            if norm(s_ml(iterea)-s_c(iterea),1)>1e-2
                sumk=sumk+1;
            end
        end
        ser_ml(snr_pt)=ser_ml(snr_pt)+sumk;
        
        ser_ldr_lra_df(snr_pt)=ser_ldr_lra_df(snr_pt)+sum(sc_ldr_lra_df~=s_c);
        ser_ldr_ld(snr_pt)=ser_ldr_ld(snr_pt)+sum(sc_ldr_ld~=s_c);
        
        sumk=0;
        for iterea=1:N
            if norm(s_gpm(iterea)-s_c(iterea),1)>1e-2
                sumk=sumk+1;
            end
        end
            
        
        ser_gpm(snr_pt)=ser_gpm(snr_pt)+sumk;
        
    end
    %compute the SER
ser_zf_df(snr_pt)=ser_zf_df(snr_pt)/(max_no_simulations*N);

ser_lra_mmse_df(snr_pt)=ser_lra_mmse_df(snr_pt)/(max_no_simulations*N);
ser_nld(snr_pt)=ser_nld(snr_pt)/(max_no_simulations);
ser_mmse_ld(snr_pt)=ser_mmse_ld(snr_pt)/(max_no_simulations*N);
ser_ml(snr_pt)=ser_ml(snr_pt)/(max_no_simulations*N);
ser_ldr_lra_df(snr_pt)=ser_ldr_lra_df(snr_pt)/(max_no_simulations*N);
ser_ldr_ld(snr_pt)=ser_ldr_ld(snr_pt)/(max_no_simulations*N);

ser_gpm(snr_pt)=ser_gpm(snr_pt)/(max_no_simulations*N);
    close(wb);
end

%compute the SER
% ser_zf_df=ser_zf_df/(max_no_simulations*N);
% 
% ser_lra_mmse_df=ser_lra_mmse_df/(max_no_simulations*N);
% ser_nld=ser_nld/(max_no_simulations);
% ser_mmse_ld=ser_mmse_ld/(max_no_simulations*N);
% ser_ml=ser_ml/(max_no_simulations*N);
% ser_ldr_lra_df=ser_ldr_lra_df/(max_no_simulations*N);
% ser_ldr_ld=ser_ldr_ld/(max_no_simulations*N);
% 
% ser_gpm=ser_gpm/(max_no_simulations*N);


% plot the SER'
semilogy(SNR,ser_zf_df,'--*k',SNR,ser_mmse_ld,'--^m',SNR,ser_gpm,'--pr',SNR,ser_ml,'--ob')
grid on;
legend('ZF DF','MMSE DF', 'GPM','MMSE');
xlabel('SNR (dB)'); ylabel('SER');
title(['M=' int2str(M) ' ' 'N=' int2str(N) ' ' int2str(MPSK_size) '-MPSK']);

